% Interpreter for Popeye	v1.1
%
module interpreter.

import popeyesig, javasig, list.

% Detects atomic goals
% 
type non_atomic		cmd -> o.
type atomic		cmd -> o.

non_atomic (A >> B   ).
non_atomic (empty    ).
non_atomic (rw A B C ).
non_atomic (new M C  ).
non_atomic (super A M).
non_atomic (show     ).

atomic A :- not (non_atomic A).

% Instancers
% 
type meth_instan	meth -> meth -> o.
% type cmd_instan		cmd  -> cmd  -> o.

meth_instan (poppi S G) I :- meth_instan (G A) I, !.
meth_instan (P >- Q) (P >- Q).

% cmd_instan (popfx S C) I :- cmd_instan (C A) I, !.
% cmd_instan C C.

% Interpreter
% 
type interp	list priv -> list meth -> cmd -> ( list priv -> list meth -> o ) -> o.

% Atomic goals
%
interp InS InP G K :-
  atomic G,
  memb InP Cl( meth_instan Cl (G >- Q), interp InS InP Q K ).

interp InS InP ( lamb G ) K :-
  G (K InS InP).		

% Init
% 
interp InS InP (init Name Val) K :-
  build InS (pv Name Val) OutS (
  K OutS InP ).
  
% Rw 
%
interp InS InP (rw (pcst C N) V V) K :- !,
  memb InS (pv (pcst C N) V) (K InS InP).		% Constants cannot be re-written

interp InS InP (rw Name Old New) K :-
  memb InS (pv Name Old) (
  memb_replace InS (pv Name Old) (pv Name New) OutS ( 
  K OutS InP )).

% while
%
% interp InS InP (while W) K :-
%  cmd_instan W (A >> B +> C),
%  (interp InS InP A ( 
%   OutS\ OutP\ ( interp OutS OutP B end,
%		 interp OutS OutP (C >> while W) K;
%		 not(interp OutS OutP B end),
%		 interp OutS OutP empty K )
%  )).

% Continuation
%
interp InS InP (G >> H) K :- 
  interp InS InP G OutS\ OutP\ ( interp OutS OutP H K ).

% Branch
%
interp InS InP (G +> H |> I) K :-
  (interp InS InP G end,
   interp InS InP H K );
  (not (interp InS InP G end),
   interp InS InP I K ).

% ==
%
interp InS InP (G == H) K :-
  G = H, K InS InP.

% Empty clause: always succeeds
%
interp InS InP (empty) K :- K InS InP.

% Same for type casting
% 
interp InS InP (cast A T) K :- K InS InP.

% Object creation 
%
interp InS InP (popnew M S L) K :-
  interp InS InP (new M L) K.

interp InS InP (new M L) K :-
  pi A\ ( 
  popmod M Context,
  load_context InS InP (Context A) (
  OutS\ OutP\ interp OutS OutP (cons A M >> L A) K )). 

interp InS InP (super A M) K :-
  popmod M Context,
  load_context InS InP (Context A) (
  OutS\ OutP\ interp OutS OutP (cons A M) K ).


% Show
%
interp InS InP (show) K :- term_to_string InS S,
			   print S, print "\n",
			   term_to_string InP P,
			   print P, print "\n",
			   K InS InP.

% Context loader
% 
type load_context list priv -> list meth -> prog -> (list priv -> list meth -> o) -> o.

load_context InS InP (popcon NewS NewP) K :-
  append NewS InS OutS (    
  append NewP InP OutP ( 
  K OutS OutP)).

load_context InS InP (loc C N B) K :-
  pi A\ load_context InS InP (B A) K.

load_context InS InP (poploc S C N B) K :-
  load_context InS InP (loc C N B) K.

load_context InS InP (sta C N V Prog) K :-
  build InS (pv (pcst C N) V) OutS (
  load_context OutS InP Prog K ).

load_context InS InP (popsta S C N V Prog) K :-
  load_context InS InP (sta C N V Prog) K.

% The final step performed by the interpreter
%
type end 	list priv -> list meth -> o.
end _ _.

% Support for built in types   
%
interp InS InP ( (pint A) p< (pint B) ) K :-
	A < B, K InS InP.	

interp InS InP ( (pint A) p> (pint B) ) K :-
	A > B, K InS InP.

interp InS InP ( inc (pint A) (pint B) ) K :-
	B is A + 1, K Ins InP.

interp InS InP ( dec (pint A) (pint B) ) K :-
	B is A - 1, K Ins InP.
