module popeyesig.

% Lambda Prolog types for Popeye expressions
%
kind 	ref		type. 			% For references
kind 	cst		type.			% For constants
type 	pcst		Any	-> cst	-> ref.

% Interpreter types
%
kind 	cmd		type.
kind 	meth		type.
kind 	prog		type.
kind 	priv		type.
kind 	class		type.

% Popeye connectives
%
type 	>- 		cmd	-> cmd 	-> meth.	% declaration
type	>>		cmd	-> cmd 	-> cmd.		% continuation
type	+>		cmd	-> cmd 	-> cmd.		% conditionnal "if"
type 	|>		cmd	-> cmd 	-> cmd.		% conditionnal "else"

infixl	>-	1.
infixr	>>	4.
infixr  +>	3.	
infixr  |>	2.

% Interpreter constructs
%
type 	popmod		class	  ->	(Any -> prog) 	-> o.
type 	poploc		string	  -> 	Any -> ref	-> (Bny -> prog)	-> prog.
type 	   loc				Any -> ref	-> (Bny -> prog)	-> prog.
type 	popsta		string	  -> 	Any		-> cst		-> ref	-> prog	-> prog.
type 	   sta				Any		-> cst		-> ref 	-> prog -> prog.
type 	popcon		list priv -> 	list meth 	-> prog.

% The binder for simulating the linear context
%
type 	pv		ref	-> ref	  -> priv.

% Labeled universal quantifiers for guarding logic variables
%
type 	poppi		string -> (Any -> meth) -> meth.
% type 	popfx		string -> (Any -> cmd)  -> cmd.

% Popeye keywords
%
type 	rw		ref 	-> ref 	-> ref 	-> cmd.			% Private access
type	init		ref 	-> ref 	-> cmd.				% Private init

type	new		class 			-> (Any -> cmd)	-> cmd. % New
type 	popnew		class	-> string	-> (Any -> cmd) -> cmd.	% Named new
type	cons		ref 	-> class 	-> cmd.			% Constructor
type 	super		ref 	-> class 	-> cmd.			% Inheritance

type 	==		ref 	-> ref	-> cmd.
infixr  == 		5.

type 	empty		cmd.

% Usefull stuff
%
type 	show 		cmd.

% A little hack that converts LProlog definitions WITH continuations into 
% valid Popeye commands.
%
type 	lamb 		(o -> o) -> cmd.
type 	top		o.
top.

% Types
%
kind 	pt		type.

type 	cast		ref 	-> pt	-> cmd.

%% TEMP

% Tracer
%
type tprint 	list A -> o.
type vprint	A -> o.

tprint Theta :- term_to_string Theta Theta_S, print Theta_S.
vprint Var :- term_to_string Var Var_S, print Var_S.





